freeze;

declare attributes FldCom: Witt_ring;

//////////////////////////////////////////////////
// IsSumOfSquares
// (just for completeness)
intrinsic IsSumOfSquares( a::FldComElt ) -> BoolElt
{
Returns true iff a is a sum of squares
}
    return true;
end intrinsic;

intrinsic IsSOS( a::FldComElt ) -> BoolElt
{ " }
    return IsSumOfSquares(a);
end intrinsic;


//////////////////////////////////////////////////
// LengthOfSOS
// (just for completeness)
intrinsic LengthOfSOS( a::FldComElt ) -> RngIntElt
{
Given a nonzero element a, this function determines
minimal number of summands needed to express a as a sum of squares. 
It returns 0 if a is not a sum of squares
}
    return 1;
end intrinsic;

intrinsic LengthOfSumOfSquares( a::FldComElt ) -> RngIntElt
{ " } //"
    return LengthOfSOS(a);
end intrinsic;

//////////////////////////////////////////////////
// Level
// (just for completeness)
intrinsic Level( K::FldCom ) -> RngIntElt
{    
This function comutes the level s(K) of a number field K,
i.e. a lowest number s such that -1 is a sum of s squares
it returns 0 if K is formally real.
}
    return 1;
end intrinsic;


//////////////////////////////////////////////////
// IsFormallyReal
// just for completeness
intrinsic IsFormallyReal( K::FldRat ) -> BoolElt
{
Returns true iff K is formally real (i.e. -1 is not a sum of squares)
}
    return false;
end intrinsic;

intrinsic IsReal( K::FldRat ) -> BoolElt
{ " } //"
    return false;
end intrinsic;


//////////////////////////////////////////////////
// PythagorasNumber & PythagorasElement
// just for completeness
intrinsic PythagorasNumber( K::FldCom ) -> RngIntElt
{
This function computes the Pythagoras number P(K) of a number field K,
i.e. the lowest number p such that every sum of squares in K
is a sum of p squares.
}
    return 1;
end intrinsic;

intrinsic PythagorasElement( K::FldCom ) -> FldNumElt
{
Returns an element a of K whose length equals the Pythagoras 
number P(K) of K.
}
    return 1;
end intrinsic;


function is_isotropic_CC( q )
    // Given a quadratic form q with complex coefficients,
    // this function tests if q is isotropic.

    return Dimension(q) gt 1;
end function;

function is_hyperbolic_CC( q )
    // Given a quadratic form q with real coefficients,
    // this function tests if q is isotropic.

    return IsEven(Dimension(q));
end function;

function anisotropic_dimension_CC( q )
// Given a quadratic form q over the complex numbers, 
// this function returns a dimension of an anisotropic part of q.
    return IsEven(Dimension(q)) select 0 else 1;
end function;


//////////////////////////////////////////////////
// Witt ring

intrinsic WittRing( F::FldCom ) -> RngWittQF
{
Constructs the Witt ring of similarity classes of quadratic forms over F.
}
    if not assigned F`Witt_ring then
        W := New(RngWittQF);
        W`base_ring := F;
        W`is_explicit := true;
        W`explicit_model := Integers(2);
        F`Witt_ring := W;
    end if;
    return F`Witt_ring;
end intrinsic;

//////////////////////////////////////////////////
// IsTorsion
// just for completeness
function is_torsion_CC( _q )
// Check if the element _q of a Witt ring is torsion.
    return true;
end function;

