freeze;

declare attributes FldRe: Witt_ring;

//////////////////////////////////////////////////
// IsSumOfSquares
// (just for completeness)
intrinsic IsSumOfSquares( a::FldReElt ) -> BoolElt
{
Returns true iff a is a sum of squares
}
    return a ge 0;
end intrinsic;

intrinsic IsSOS( a::FldReElt ) -> BoolElt
{ " } //"
    return IsSumOfSquares(a);
end intrinsic;


//////////////////////////////////////////////////
// LengthOfSOS
// (just for completeness)
intrinsic LengthOfSOS( a::FldReElt ) -> RngIntElt
{
Given a nonzero element a, this function determines
minimal number of summands needed to express a as a sum of squares. 
It returns 0 if a is not a sum of squares
}
    return IsSOS(a) select 1 else 0;
end intrinsic;

intrinsic LengthOfSumOfSquares( a::FldReElt ) -> RngIntElt
{ " } //"
    return LengthOfSOS(a);
end intrinsic;


//////////////////////////////////////////////////
// Level
// (just for completeness)
intrinsic Level( K::FldRe ) -> RngIntElt
{    
This function comutes the level s(K) of a number field K,
i.e. a lowest number s such that -1 is a sum of s squares
it returns 0 if K is formally real.
}
    return 0;
end intrinsic;


//////////////////////////////////////////////////
// Height
// (just for completeness)
intrinsic Height( K::FldRe ) -> RngIntElt
{
This function computes the height h(K) of a number field K,
i.e. exp( Wt(K) ), where Wt(K) is denotes the torsion subgroup
of the Witt ring of K.
}
    return 1;
end intrinsic;



//////////////////////////////////////////////////
// IsFormallyReal
// just for completeness
intrinsic IsFormallyReal( K::FldRat ) -> BoolElt
{
Returns true iff K is formally real (i.e. -1 is not a sum of squares)
}
    return true;
end intrinsic;

intrinsic IsReal( K::FldRat ) -> BoolElt
{ " } //"
    return true;
end intrinsic;


//////////////////////////////////////////////////
// PythagorasNumber & PythagorasElement
// just for completeness
intrinsic PythagorasNumber( K::FldRe ) -> RngIntElt
{
This function computes the Pythagoras number P(K) of a number field K,
i.e. the lowest number p such that every sum of squares in K
is a sum of p squares.
}
    return 1;
end intrinsic;

intrinsic PythagorasElement( K::FldRe ) -> FldNumElt
{
Returns an element a of K whose length equals the Pythagoras 
number P(K) of K.
}
    return 1;
end intrinsic;


function is_isotropic_RR( q )
    // Given a quadratic form q with real coefficients,
    // this function tests if q is isotropic.

    return Abs(&+ [ Sign(c) : c in q`coeffs ]) lt Dimension(q);
end function;

function is_hyperbolic_RR( q )
    // Given a quadratic form q with real coefficients,
    // this function tests if q is isotropic.

    return IsZero(&+ [ Sign(c) : c in q`coeffs ]);
end function;

function anisotropic_dimension_RR( q )
// Given a quadratic form q over the reals, 
// this function returns a dimension of an anisotropic part of q.
    return Abs( &+ [ Sign(c) : c in q`coeffs ] );
end function;


//////////////////////////////////////////////////
// IsTorsion

function is_torsion_RR( _q )
// Check if the element _q of a Witt ring is torsion.
    return IsZero(_q);
end function;


//////////////////////////////////////////////////
// Witt ring

intrinsic WittRing( F::FldRe ) -> RngWittQF
{
Constructs the Witt ring of similarity classes of quadratic forms over F.
}
    if not assigned F`Witt_ring then
        W := New(RngWittQF);
        W`base_ring := F;
        W`is_explicit := true;
        W`explicit_model := Integers();
        F`Witt_ring := W;
    end if;
    return F`Witt_ring;
end intrinsic;

